var async    = require('async');
var canvass;
module.exports = {
    '@tags': ['cf_copyPaste'],
    before: function (client) {
        client
            .url(client.launchUrl)
            .waitForElementVisible('body', 1000);
    },

    after: function (client) {

        client.end();
    },
    /**
     *  Test: 2-001
     *  Expects:
     *      - Node should be added in the canvass
     */
    "2-001": function (client) {
        async.series([
                //load HARK Designer
                function (callback) {
                    canvass = client.page.canvass();
                    canvass.navigate();

                    //check if body is now visible
                    canvass.expect.element('body').to.be.present;

                    //should have Main sheet
                    canvass.expect.element('@mainSheet').to.be.visible;
                    callback();
                },
                //add one node to canvass
                function (callback) {
                    canvass.addNodeByName('NNETExec', function () {
                        canvass.expect.element('@nodeDOM').to.be.visible;
                        callback();
                    });
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 2-002
     *  Expects:
     *      - Node should be copied and stored in the localStorage.
     *      - To check the localStorage please see it in the inspect element
     *  Remarks:
     *      - No direct way to access localStorage
     */
    "2-002": function (client) {
        async.series([
                //select MAIN sheet
                function (callback) {
                    canvass.selectSheet('@mainSheetTab', function () {
                        canvass.expect.element('@nodeDOM').to.be.visible;
                        callback();
                    });
                },
                //node should be selected
                function (callback) {
                    canvass.selectNode('@nodeDOM', function(){
                        canvass.expect.element('@nodeDOM').to.have.attribute('class').contains('nodeSelected');
                    });

                    callback();
                },
                //copy node via Ctrl+C
                function (callback) {
                    canvass.copyNode(client, 'html > body > div#container > div#sheets > div:nth-of-type(1) > div:nth-of-type(1)', 2, function(){
                        callback();
                    });
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 2-003
     *  Expects:
     *      - Copied node should be pasted in the sheet selected
     */
    "2-003": function (client) {
        async.series([
                //paste node via Ctrl+V
                function (callback) {
                    canvass.paste(client, 'html > body > div#container > div#sheets > div:nth-of-type(1)', 2, function(){
                        callback();
                    });
                },
                //check if a second node is added
                function (callback) {
                    canvass.expect.element('@nodeDOM2').to.be.visible;
                    callback();
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 2-004
     *  Expects:
     *      - Copied Node should be pasted in the other sheet
     */
    "2-004": function (client) {
        async.series([
                //Create new tab
                function (callback) {
                    canvass.addSheet('test1', function(){
                        callback();
                    });
                },
                //check if new tab is added
                function (callback) {
                    canvass.expect.element('@sheet1').to.be.visible;
                    callback();
                },
                //paste node via Ctrl+V
                function (callback) {
                    canvass.paste(client, 'html > body > div#container > div#sheets > div:nth-of-type(1)', 2, function(){
                        callback();
                    });
                },
                //check if a third node is added
                function (callback) {
                    canvass.expect.element('@nodeDOM3').to.be.visible;
                    callback();
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 2-005
     *  Expects:
     *      - Nodes should be selected
     *      - Nodes should have shadow as indicator that it should be selected.
     */
    "2-005": function (client) {
        async.series([
                //add another node
                function (callback) {
                    canvass.addNodeByName('NNETExec', function(){
                        callback();
                    });
                },
                //add another node
                function(callback) {
                    //check if node is added
                    canvass.expect.element('@nodeDOM4').to.be.visible;

                    //add another node
                    canvass.addNodeByName('NNETExec', function(){
                        callback();
                    });
                },
                //add another node
                function(callback) {
                    //check if node is added
                    canvass.expect.element('@nodeDOM5').to.be.visible;
                    callback();
                },
                //select all nodes
                function (callback) {
                    canvass.selectNodes(client, 'html > body > div#container > div#sheets > div:nth-of-type(2)', function(){
                        callback();
                    });
                },
                //check if all nodes are selected
                function (callback) {
                    //check if node 3 is selected
                    canvass.expect.element('@nodeDOM3').to.have.attribute('class').contains('nodeSelected');
                    //check if node 4 is selected
                    canvass.expect.element('@nodeDOM4').to.have.attribute('class').contains('nodeSelected');
                    //check if node 5 is selected
                    canvass.expect.element('@nodeDOM5').to.have.attribute('class').contains('nodeSelected');

                    callback();
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 2-006
     *  Expects:
     *      - Node should be copied and stored in the localStorage.
     *      - To check the localStorage please see it in the inspect element
     *  Remarks:
     *      - No direct way to access localStorage
     */
    "2-006": function (client) {
        async.series([
                //copy all selected nodes
                function (callback) {
                    canvass.copyNode(client, 'html > body > div#container > div#sheets > div:nth-of-type(2) > div:nth-of-type(1)', 2, function(){
                        callback();
                    });
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 2-007
     *  Expects:
     *      - Copied nodes should be pasted in the sheet selected
     *      - Links should be drawn if the nodes selected have a link.
     */
    "2-007": function (client) {
        async.series([
                //paste nodes
                function (callback) {
                    canvass.paste(client, 'html > body > div#container > div#sheets > div:nth-of-type(2) > div:nth-of-type(1)', 2, function(){
                        callback();
                    });
                },
                //check if a nodes are added
                function (callback) {
                    //check if a sixth node is added
                    canvass.expect.element('@nodeDOM6').to.be.visible;
                    //check if a seventh node is added
                    canvass.expect.element('@nodeDOM7').to.be.visible;
                    //check if a eight node is added
                    canvass.expect.element('@nodeDOM8').to.be.visible;

                    callback();
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 2-008
     *  Expects:
     *      - Copied Nodes should be pasted in the other sheet.
     *      - Links should be drawn if the nodes selected have a link.
     */
    "2-008": function (client) {
        async.series([
                //Create new tab
                function (callback) {
                    canvass.addSheet('test2', function(){
                        callback();
                    });
                },
                //check if new tab is added
                function (callback) {
                    canvass.expect.element('@sheet2').to.be.visible;
                    callback();
                },
                //paste nodes
                function (callback) {
                    canvass.paste(client, 'html > body > div#container > div#sheets > div:nth-of-type(2) > div:nth-of-type(1)', 2, function(){
                        callback();
                    });
                },
                //check if nodes are added
                function (callback) {
                    canvass.expect.element('@nodeDOM9').to.be.visible;
                    canvass.expect.element('@nodeDOM10').to.be.visible;
                    canvass.expect.element('@nodeDOM11').to.be.visible;
                    callback();
                }
            ],
            function () {
                //Finished
            });
    }
};