var async    = require('async');

//text value
var textValue = 'Sample Text';
var textValue2 = 'This\nis\na\nsample\nnote\nedit';

var canvass;

module.exports = {
    '@tags': ['cf_note'],
    before: function (client) {
        client
            .url(client.launchUrl)
            .waitForElementVisible('body', 1000);
    },

    after: function (client) {

        client.end();
    },
    /**
     *  Test: 4-001
     *  Expects:
     *      - Load designer should load an empty canvass
     */
    "4-001": function (client) {
        async.series([
                //load HARK Designer
                function (callback) {
                    canvass = client.page.canvass();
                    canvass.navigate();

                    //check if body is now visible
                    canvass.expect.element('body').to.be.present;

                    callback();
                },
                //check if MAIN sheet is created
                function (callback) {
                    //should have Main sheet
                    canvass.expect.element('@mainSheet').to.be.visible;
                    callback();
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 4-002
     *  Expects:
     *      - Context menu will be displayed.
     *      - If the canvass is loaded freshly, then there will be only one item on the context menu which is the "Add note" item.
     *      - If the user stored clipboard to the local storage by copying a node or note there will be multiple items in the context menu.
     */
    "4-002": function (client) {
        async.series([
                //pause to make sure that HARK Designer is fully loaded
                function (callback) {
                    canvass.api.pause(2000, function () {
                        callback();
                    });
                },
                //click MAIN Sheet
                function (callback) {
                    canvass.click('@mainSheet', function () {
                        callback();
                    });
                },
                //right click on the canvass
                function (callback) {
                    client
                        .moveToElement('#canvas2_MAIN', 100, 100)
                        .mouseButtonClick('right', function(){
                            callback();
                        });
                },
                function (callback) {
                    canvass.waitForElementPresent('@mainContextMenu', 2000, function(){
                        //check if context menu is displayed
                        canvass.expect.element('@mainContextMenu').to.be.visible;
                        //check if Add Note option is displayed
                        canvass.expect.element('@addNoteOption').to.be.visible;

                        callback();
                    });
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 4-003
     *  Expects:
     *      - Note element with the content "Sample Text" should be drawn in the sheet.
     *      - Note element should be layered always on the top of the nodes.
     *      - Note should be draggable in the canvass.
     *      - Note should be resizable.
     */
    "4-003": function (client) {
        async.series([
                //click Add Note Option
                function (callback) {
                    canvass.click('@addNoteOption', function () {
                        callback();
                    });

                },
                //set text for Note
                function (callback) {
                    canvass.waitForElementVisible('@addNote0Dialog', 1000)
                        .setValue('@addNote0DialogTextArea', textValue)
                        .click('@addNote0DialogOkButton', function(){

                            callback();
                        });
                },
                function (callback) {
                    //note 1 should be created
                    canvass.expect.element('@note1').to.be.present;
                    //note text should be correct
                    canvass.expect.element('@note1').text.to.equal(textValue);
                    callback();
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 4-004
     *  Expects:
     *      - Context menu will be displayed.
     *      - Context menu items:
     *          1. Delete note
     *          2. Copy note
     */
    "4-004": function (client) {
        async.series([
                //right click on the note
                function (callback) {
                    client
                        .moveToElement('#note0', 10, 10)
                        .mouseButtonClick('right', function(){
                            callback();
                        });
                },
                function (callback) {
                    canvass.waitForElementPresent('@noteContextMenu', 2000, function(){
                        //check if context menu is displayed
                        canvass.expect.element('@noteContextMenu').to.be.visible;
                        //check if Copy Note option is displayed
                        canvass.expect.element('@noteCopyOption').to.be.visible;
                        //check if Delete Note option is displayed
                        canvass.expect.element('@noteDeleteOption').to.be.visible;

                        callback();
                    });
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 4-005
     *  Expects:
     *      - Context menu should be closed
     */
    "4-005": function (client) {
        async.series([
                //click Copy Note Option
                function (callback) {
                    canvass.click('@noteCopyOption', function () {
                        callback();
                    });

                },
                //context menu should be closed
                function (callback) {
                    canvass.waitForElementNotPresent('@noteContextMenu', 1000, function () {
                        canvass.expect.element('@noteContextMenu').to.not.be.present;
                        callback();
                    });
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 4-006
     *  Expects:
     *      - It should be drawn in the canvass (active sheet).
     *      - The note content should be similar to the original note copied.
     *      - Copied note can also be pasted on another tab(instance of Hark Designer) in the same browser.
     *      - Note can be draggable in the canvass.
     */
    "4-006": function (client) {
        async.series([
                //paste Note
                function (callback) {
                    canvass.paste(client, 'html > body > div#container > div#sheets > div:nth-of-type(1)', function(){
                        callback();
                    });
                },
                //another note should be created
                function (callback) {
                    //note 2 should be created
                    canvass.expect.element('@note2').to.be.present;
                    //note text should be same with note 1
                    canvass.expect.element('@note2').text.to.equal(textValue);

                    callback();
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 4-007
     *  Expects:
     *      - "Edit note" will be displayed.
     */
    "4-007": function (client) {
        async.series([
                //double click on second note
                function (callback) {
                    client
                        .moveToElement('#note1', 0, 0)
                        .doubleClick(function(){
                            callback();
                        });
                },
                //Edit Note Dialog should appear
                function (callback) {
                    canvass.waitForElementVisible('@addNote1Dialog', 1000, function () {
                        canvass.expect.element('@addNote1Dialog').to.be.present;
                        callback();
                    });
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 4-008
     *  Expects:
     *      - Note content will be updated to
     *           "This
     *           is
     *           a
     *           sample
     *           note
     *           edit".
     *      - Note element should be resizable.
     *      - Note element should have scroll bar if the text is more than two lines.
     */
    "4-008": function (client) {
        async.series([
                //select note text area
                function (callback) {
                    canvass.click('@addNote1DialogTextArea', function(){
                        callback();
                    });
                },
                //Clear note text area
                function (callback) {
                    client.keys([client.Keys.CONTROL, "a"])
                        .keys(client.Keys.NULL)
                        .keys(client.Keys.DELETE)
                        .keys(client.Keys.NULL, function () {
                            callback();
                        });
                },
                //set new note text
                function (callback) {
                    canvass.setValue('@addNote1DialogTextArea', textValue2, function () {
                        callback();
                    })
                },
                //click Ok button
                function (callback) {
                    canvass.click('@addNote1DialogOkButton', function(){
                        callback();
                    });
                },
                //note dialog should be closed
                function (callback) {
                    canvass.waitForElementNotVisible('@addNote1Dialog', 1000, function () {
                        callback();
                    });
                },
                //note text should be updated
                function (callback) {
                    canvass.expect.element('@note2').text.to.equal(textValue2);
                    callback();
                },
                //drag note
                function (callback) {
                    canvass.dragNodeOrNote(client, '#note0', 300, 300, function () {
                        callback();
                    });
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 4-009
     *  Expects:
     *      - Selected note should be deleted/removed in the canvass.
     */
    "4-009": function (client) {
        async.series([
                //deselect second note
                function (callback) {
                    canvass.click('@mainSheet', function(){
                        callback();
                    });
                },
                //delete note
                function (callback) {
                    canvass.deleteNote(client, '#note0', function(){
                        callback();
                    });
                },
                //note should be deleted
                function (callback) {
                    canvass.expect.element('@note1').to.be.not.present;

                    callback();
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 4-010
     *  Expects:
     *      - Network with the note will be saved.
     */
    "4-010": function (client) {
        async.series([
                //save network
                function (callback) {
                    canvass.saveNetwork(client, 'sample_note.n', function () {
                        callback();
                    });
                },
                //save dialog should be closed
                function (callback) {
                    canvass.expect.element('@saveSheetDialog').to.not.be.visible;

                    callback();
                }
            ],
            function () {
                //Finished
            });
    },
    /**
     *  Test: 4-011
     *  Expects:
     *      - The saved network "sample_note.n" will be loaded in the Hark Designer.
     *      - Note content will be updated to
     *           "This
     *           is
     *           a
     *           sample
     *           note
     *           edit".
     *      - Note element should be resizable.
     *      - Note element should have scroll bar if the text is more than two lines.
     */
    "4-011": function (client) {
        async.series([
                //load network
                function (callback) {
                    canvass.loadNetwork(client, 'sample_note.n', function () {
                        callback();
                    });
                },
                //save dialog should be closed and note text should be correct
                function (callback) {
                    canvass.expect.element('@fileDialog').to.not.be.visible;
                    canvass.expect.element('@note2').text.to.equal(textValue2);

                    callback();
                }
            ],
            function () {
                //Finished
            });
    }
};